:- use_package([dcg,assertions]).

%----------------------------------------------------------------------
:- comment(title, "A .aux parser in Prolog").
:- comment(author, "Manuel Hermenegildo").

:- comment(module, "A simple parser for .aux files in Prolog. Simply
   to gather the citations. This is just a first shot, but does a
   reasonable job and should be easily extensible.").

%----------------------------------------------------------------------

auxparser(Refs,RefsT) --> 
	spaces,
 	"\\citation",
 	open,
  	parse_refs(Refs,RefsT),
 	close,
	spaces.
%% 	eol.
%% %% 	{name(AKey,Key),
%% %% 	 append(AVPL,[[]],Rest),
%% %%          to_lower_case(Type,LCType)}.
auxparser(T,T,_,_).

parse_refs(Refs,RefsT) --> 
	spaces,
	chars_no_space(Refs,IT),
	spaces, 
	comma,
	spaces, 
	parse_refs(IT,RefsT).
parse_refs(Refs,RefsT)    --> 
	spaces,
	chars_no_space(Refs,RefsT),
	spaces.

spaces --> space, spaces.
spaces --> [].

chars_no_space(H,T) --> char_no_space(H,IT), chars_no_space(IT,T).
chars_no_space(T,T)    --> [].

char_no_space([X|T],T) --> [X], 
	{\+ space([X],[]), \+ curly_char([X],[]), \+ comma([X],[]) }.

curly_char --> open.
curly_char --> close.

comma      --> [0',].
open       --> [0'{].
close      --> [0'}].
digit(X)   --> [X], {X > 47, X<58}.
space      --> [X], {X=<32}. % covers EOF, ^I, ^G, ^J, ^K, ^L, ^M, ^Z, ESC, ^_
eol        --> [0'
].                         % newline (10)

%----------------------------------------------------------------------
:- comment(version_maintenance,dir(version)).

:- comment(version(0*4+58,2004/03/05,12:31*27+'CET'), "Created
   auxparser from bibparser.  (Manuel Hermenegildo)").

%----------------------------------------------------------------------


