:- use_package([assertions,fsyntax]).

%% CIAO-specific
:- use_module(library(format)).
:- use_module(library(ttyout)).

%----------------------------------------------------------------------
% Format a paper for latex itemize, now limited multilingual support 
%----------------------------------------------------------------------

:- ensure_loaded(support).

% Format common info, English default

:- push_prolog_flag(multi_arity_warnings,off).

pp_papers_latex(Papers) :-
	pp_papers_latex(Papers,eng).

pp_papers_latex(Papers,Lang) :-
%% 	format(user_error,"[Writing out papers",[]),ttyflush,
	do_pp_papers_latex(Papers,0,Lang).

:- pop_prolog_flag(multi_arity_warnings).

do_pp_papers_latex([],_Count,_Lang).
%% 	format(user_error," done. ~w entries processed.]~n",[Count]),ttyflush.
do_pp_papers_latex([H|T],Count,Lang) :-
        pp_paper_latex(H,Lang),
	!,
 	format(user_error,".",[]),ttyflush,
	NCount is Count + 1,
	do_pp_papers_latex(T,NCount,Lang).
do_pp_papers_latex([prolog_paper(_Y,_M,K,_L)|T],Count,Lang) :- 
	format(user_error,"[ERROR: could not format ~w",[K]),
	do_pp_papers_latex(T,Count,Lang).

pp_paper_latex(prolog_paper(_Year,_Month,Keyword,List),Lang) :-
	member([butype,Type],List),
	format('\\item ',[]),
%% 	format('\\nocite{~w} \\ ',[Keyword]),
        print_item(author,List,Keyword,". ",Lang),
	format('{\\em ',[]),
	print_item(title,List,Keyword,".",Lang),
	format('}~n',[]),
	print_special_info(Type,List,Keyword,Lang),
	print_item_optional(publisher,List,Keyword,", ",Lang),
        %% In these cases series already taken care of 
	(  (Type \== "manual",
	    Type \== "proceedings",
	    Type \== "editedbook",
	    Type \== "book",
	    Type \== "workshop",
	    Type \== "incollection",
	    Type \== "invited",
	    Type \== "inproceedings",
	    Type \== "techreport", 
	    Type \== "article")
        -> print_item_optional(series,List,Keyword,", ",Lang)
        ;  true ),
        %% In these cases number already taken care of
	(  (Type \== "manual",
	    Type \== "proceedings",
	    Type \== "editedbook",
	    Type \== "book",
	    Type \== "workshop",
	    Type \== "incollection",
	    Type \== "invited",
	    Type \== "inproceedings",
	    Type \== "techreport", 
	    Type \== "article")
        -> print_item_optional(number,List,Keyword,", ",Lang)
        ;  true ),
	print_item_optional(edition,List,Keyword,", ",Lang),
	(  member([publisher,Publisher],List),
	   member([institution,Institution],List),
	   Publisher = Institution
	-> true
	;  print_item_optional(institution,List,Keyword,", ",Lang)),
	(  member([publisher,Publisher],List),
	   member([organization,Organization],List),
	   Publisher = Organization
	-> true
	;  print_item_optional(organization,List,Keyword,", ",Lang)),
	( Type = "article"
        -> print_item_optional(month,List,Keyword," ",Lang)
        ;  print_item(month,List,Keyword," ",Lang)),
        print_item(year,List,Keyword,".",Lang),
        print_item_optional(note,List,Keyword,".",Lang),
	format('~n',[]),
	ttyflush.

% Format a field

print_item_optional(Item,List,Keyword,Sep,Lang) :-
	print_item_common(Item,List,Keyword,Sep,Lang),
	!.
print_item_optional(_Item,_List,_Keyword,_Sep,_Lang).

print_item(Item,List,Keyword,Sep,Lang) :-
	print_item_common(Item,List,Keyword,Sep,Lang),
	!.
print_item(Item,_List,Keyword,_Sep,_Lang) :-
	format(user_error,'[WARNING: No ~a info in DB for ~a]~n',
                           [Item,Keyword]).

print_item_common(pages,List,Keyword,Sep,Lang):-
	get_page_number(Keyword,List,Number,In,En),
	!,
	misc_text(pages_long,Lang,PagesText),
	(In == [] -> 	
           ( number(Number) ->
	     format('~w',[Number]),
%	     format(' pages~s~n',[Sep])
	     format(' ~s~s~n',[PagesText,Sep])
	   ; true )
%	;  format(' pages ~d--~d~s~n',[In,En,Sep])
	;  format(' ~s ~d--~d~s~n',[PagesText,In,En,Sep])
	). 
%% Added to support month translation
print_item_common(month,List,_Keyword,Sep,Lang) :-
	member([month,Month],List),
	!,
	month_to_lang(Month,Lang,TranslatedMonth),
	do_print_item(month,TranslatedMonth),
	format("~s~n",[Sep]).
print_item_common(volume,List,_Keyword,Sep,Lang) :-
	member([volume,Volume],List),
	!,
	misc_text(volume_short,Lang,VolShort),
	format("~s ~s~s~n",[VolShort,Volume,Sep]).
print_item_common(number,List,_Keyword,Sep,Lang) :-
	member([number,Number],List),
	!,
	misc_text(number_short,Lang,NumShort),
	format("~s ~s~s~n",[NumShort,Number,Sep]).
print_item_common(title,List,Keyword,Sep,_Lang) :-
	member([title,Title],List),
	!,
	(  ( member([url,MAINURL],List), AUXURL=[]; 
	     exists_www_copy(Keyword,MAINURL,AUXURL) )
	-> add_html2latex_link(MAINURL,AUXURL,Title,LTitle)
        ;  LTitle = Title),
	do_print_item(title,LTitle),
	format("~s~n",[Sep]).
print_item_common(booktitle,List,_Keyword,Sep,_Lang) :-
	member([booktitle,Title],List),
	!,
	(  member([bookurl,URL],List) 
	-> add_html2latex_link(URL,Title,LTitle)
        ;  LTitle = Title),
	do_print_item(booktitle,LTitle),
	format("~s~n",[Sep]).
print_item_common(Item,List,_Keyword,Sep,_Lang) :-
	member([Item,AList],List),
	!,
	do_print_item(Item,AList),
	format("~s~n",[Sep]).

do_print_item(I,[A]):-
	(I = author; I = editor),
	!,
        possibly_add_person_url(A,URLA),
	format('~s',[URLA]).
do_print_item(I,[A|T]):-
	(I = author; I = editor),
	!,
        possibly_add_person_url(A,URLA),
	format('~s, ',[URLA]),
        do_print_item(I,T).
do_print_item(_,A):-
	format('~s',[A]).

possibly_add_person_url(A,URLA) :-
	people_url_info_available(yes),
	add_url(A,URLA).
possibly_add_person_url(A,A).

% Format specialized info

print_special_info("manual",List,Keyword,Lang) :- !,
	print_special_info("book",List,Keyword,Lang).
print_special_info("proceedings",List,Keyword,Lang) :- !,
	print_special_info("book",List,Keyword,Lang).
print_special_info("editedbook",List,Keyword,Lang) :- !,
	print_special_info("book",List,Keyword,Lang).
print_special_info("book",List,Keyword,Lang):- 
	!,
	print_item_optional(series,List,Keyword,", ",Lang),
	print_item_optional(volume,List,Keyword,", ",Lang),
	print_item_optional(number,List,Keyword,", ",Lang),
	print_item_optional(pages,List,Keyword,", ",Lang). 

print_special_info("workshop",List,Keyword,Lang) :- !,
	print_item(booktitle,List,Keyword,", ",Lang),
	print_item_optional(series,List,Keyword,", ",Lang),
	print_item_optional(volume,List,Keyword,", ",Lang),
	print_item_optional(number,List,Keyword,", ",Lang),
	print_item_optional(pages,List,Keyword,", ",Lang).

print_special_info("incollection",List,Keyword,Lang) :- !,
	print_special_info("inproceedings",List,Keyword,Lang).
print_special_info("invited",List,Keyword,Lang) :- !,
	print_special_info("inproceedings",List,Keyword,Lang).
print_special_info("invitedtalk",List,Keyword,Lang) :- !,
	print_special_info("inproceedings",List,Keyword,Lang).
print_special_info("inproceedings",List,Keyword,Lang) :- !,
	print_item(booktitle,List,Keyword,", ",Lang),
	print_item_optional(series,List,Keyword,", ",Lang),
	print_item_optional(volume,List,Keyword,", ",Lang),
	print_item_optional(number,List,Keyword,", ",Lang),
	print_item(pages,List,Keyword,", ",Lang).

print_special_info("techreport",List,Keyword,Lang) :- !,
	print_item(number,List,Keyword,", ",Lang),
	print_item_optional(pages,List,Keyword,", ",Lang). 

print_special_info("article",List,Keyword,Lang) :- !,
	print_item(journal,List,Keyword,", ",Lang),
        ( member([volume,_],List),!,
 	  print_item(volume,List,Keyword,", ",Lang),
 	  print_item(number,List,Keyword,", ",Lang),
          print_item(pages,List,Keyword,", ",Lang)
        ;
%         format(' in press, ',[]) 
  	  misc_text(inpress,Lang,InPressText),
          format(' ~s, ',[InPressText]) 
        ).

print_special_info("mastersthesis",List,Keyword,Lang) :- !,
%       format("Ms.\\ Thesis, ",[]),
	misc_text(msthesis,Lang,MsText),
        format("~s, ",[MsText]),
        print_item(school,List,Keyword,", ",Lang),
	print_item_optional(address,List,Keyword,", ",Lang),
	print_item_optional(pages,List,Keyword,", ",Lang). 
print_special_info("phdthesis",List,Keyword,Lang) :- !,
%       format("Ph.D.\\ Thesis, ",[]),
	misc_text(phdthesis,Lang,PhDText),
        format("~s, ",[PhDText]),
        print_item(school,List,Keyword,", ",Lang),
	print_item_optional(address,List,Keyword,", ",Lang),
	print_item_optional(pages,List,Keyword,", ",Lang). 

print_special_info(Type,_List,Keyword,_Lang) :- 
	format(user_error,'[WARNING: Don\'t know how to format ~s in ~a]~n',
                           [Type,Keyword]).
	
%----------------------------------------------------------------------

%% For SICStus
comment(_,_).

:- comment(version_maintenance,dir(version)).

:- comment(version(0*4+68,2006/03/05,10:33*44+'CET'), "Renamed
   functions to fsytax.  (Manuel Hermenegildo)").

:- comment(version(0*4+59,2004/03/08,17:32*36+'CET'), "Added listing
   of invited talks with no paper (but, perhaps, slides).  (Manuel
   Hermenegildo)").

:- comment(version(0*4+51,2003/04/04,02:42*54+'CEST'), "Eliminated
   repetitions when publisher or organization are the same as
   institution.  (Manuel Hermenegildo)").

:- comment(version(0*4+47,2002/05/07,17:29*47+'CEST'), "Fixed minor
   problem with formatting of PhD Thesis in English.  (Manuel
   Hermenegildo)").

:- comment(version(0*4+46,2002/04/16,19:00*55+'CEST'), "Minor fix to
   formatting of books.  (Manuel Hermenegildo)").

:- comment(version(0*4+43,2002/04/07,16:00*26+'CEST'), "Fixed old bug
   that made series, volume, and number fields be missing from certain
   publications and sometimes repeated (incollection and book among
   others).  (Manuel Hermenegildo)").

:- comment(version(0*4+41,2002/04/06,19:54*47+'CEST'), "Changes for
   some multi-lingual support (eng/esp). Note that the name of the
   file changes from @file{pp_papers_latex_eng.pl} to
   @file{pp_papers_latex.pl} to reflect this. This may affect some
   @file{Makefile} scripts. (Manuel Hermenegildo)").

:- comment(version(0*4+34,1999/09/02,21:44*32+'MEST'), "Now 'note'
   field is included in LaTeX output (useful for 'To Appear', etc.).
   (Manuel Hermenegildo)").

:- comment(version(0*4+24,1999/03/16,14:23*51+'MET'), "Changed
   syntax/1 to use_package/1 for 0.9.  (Manuel Hermenegildo)").

:- comment(version(0*3+0,1998/06/19,10:18*10+'MET DST'), "Changed how
   paper urls are added: look for either a url in the bib database or
   the existence of a ps file (@bf{with the same name as the keyword
   of the bib entry}) in a public directory, defined in a fact
   papers_dir_url/2.  (Manuel Hermenegildo)").


%----------------------------------------------------------------------

